###################################################################################################
# zmscustom.py
#
# Implementation of class ZMSCustom (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTMLFile
import string 
import urllib
import time
import sys
# Product Imports.
from zmscontainerobject import ZMSContainerObject
from zmsobject import ZMSObject
import _blobfields
import _globals


###################################################################################################
###################################################################################################
###   
###   C o n s t r u c t o r ( s )
###   
###################################################################################################
###################################################################################################

manage_addZMSCustomForm = HTMLFile('manage_addzmscustomform', globals()) 
def manage_addZMSCustom(self, meta_id, lang, manage_lang, _sort_id, REQUEST, RESPONSE):
  """ manage_addZMSCustom """
  
  if REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
    
    ##### Create ####
    obj = ZMSCustom(self.getNewId(),_sort_id+1,meta_id)
    self._setObject(obj.id, obj)
  
    oItem = getattr(self,obj.id)
    ##### Object State ####
    oItem.setObjStateNew(REQUEST)
    ##### Init Coverage ####
    coverage = self.getDCCoverage(REQUEST)
    if lang != self.getPrimaryLanguage() or lang == coverage:
      oItem.setObjProperty('attr_dc_coverage',lang)
    ##### Init Properties ####
    oItem.manage_changeProperties(lang,manage_lang,REQUEST,RESPONSE)
    ##### VersionManager ####
    oItem.onChangeObj(REQUEST)
  
    ##### Normalize Sort-IDs ####
    self.normalizeSortIds()
  
    # Return with message.        
    message = self.getLangStr('MSG_INSERTED',manage_lang)%oItem.display_type(REQUEST)
    RESPONSE.redirect('%s/%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(self.absolute_url(),obj.id,lang,manage_lang,urllib.quote(message)))

  else:
    RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s'%(self.absolute_url(),lang,manage_lang))


###################################################################################################
###################################################################################################
###   
###   C l a s s
###   
###################################################################################################
###################################################################################################

class ZMSCustom(ZMSContainerObject): 
        
    # Properties.
    # -----------
    meta_type = "ZMSCustom"
    icon = "misc_/zms/zmsdocument.gif"
    icon_disabled = "misc_/zms/zmsdocument_disabled.gif"

    # Management Options.
    # -------------------
    manage_options = ( 
	{'label': 'TAB_EDIT',       'action': 'manage_main'},
	{'label': 'TAB_ACCESS',     'action': 'manage_users'},
	{'label': 'TAB_REFERENCES', 'action': 'manage_RefForm'},
	{'label': 'TAB_HISTORY',    'action': 'manage_UndoForm'},
	{'label': 'TAB_PREVIEW',    'action': 'preview_html'}, # empty string defaults to index_html
	)

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = (
		'manage','manage_main',
		'manage_properties','manage_changeProperties',
		'manage_moveObjUp','manage_moveObjDown',
		'manage_cutObjects','manage_copyObjects','manage_pasteObjs',
		'manage_rollbackVersionChanges','manage_requestVersionChanges',
		)
    __editorPermissions__ = (
		'manage_commitVersionChanges',
		)
    __administratorPermissions__ = (
		)
    __ac_permissions__=(
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		('ZMS Administrator', __administratorPermissions__),
		)

    # Properties.
    # -----------
    __obj_attrs__ = {
        # Changed by
        'change_uid':{'datatype':'LangString','xml':0,'lang_inherit':_globals.LI_FORBIDDEN},
        'change_dt':{'datatype':'LangDateTime','xml':0,'lang_inherit':_globals.LI_FORBIDDEN},
        # Versioned by
        'work_uid':{'datatype':'LangString','xml':0,'lang_inherit':_globals.LI_FORBIDDEN},
        'work_dt':{'datatype':'LangDateTime','xml':0,'lang_inherit':_globals.LI_FORBIDDEN},
        # Properties
        'sort_id':{'datatype':'string','xml':0},
        'active':{'datatype':'LangInt','type':'checkbox','lang_inherit':_globals.LI_OBLIGATE},
        'attr_active_start':{'datatype':'LangDateTime','lang_inherit':_globals.LI_OBLIGATE},
        'attr_active_end':{'datatype':'LangDateTime','lang_inherit':_globals.LI_OBLIGATE},
        # Meta-Data
        'attr_dc_coverage':{'datatype':'string'},
    }


    # Management Interface.
    manage_main = HTMLFile('dtml/metaobj/manage_main', globals())


    """
    ###############################################################################################    
    #
    #   CONSTRUCTOR
    #
    ###############################################################################################    
    """

    ###############################################################################################
    # ZMSCustom.__init__: 
    #
    # Constructor (initialise a new instance of ZMSCustom).
    ###############################################################################################
    def __init__(self, id='', sort_id='', meta_id=''): 
      """ ZMSCustom.__init__ """
      ZMSObject.__init__(self,id,sort_id)
      self.meta_id = meta_id


    ###############################################################################################
    # ZMSCustom.recurse_updateVersion:
    #
    # Update version.
    ###############################################################################################
    def recurse_updateVersion(self, lang, manage_lang, REQUEST):
      message = ''

      # Process super-objects.
      message = message + ZMSObject.recurse_updateVersion(self,lang,manage_lang,REQUEST)
        
      # Process object.
      # [...]

      # ---------------------------------------------------------------------
      # 20.02.2002
      # Build #061: Make object-attributes multilingual.
      # ---------------------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '061':
        attrs = []
        for attr in self.getMetaobj(self.meta_id)['attrs']:
	  if attr['type'][:3] != 'ZMS':
            attrs.append(attr['id'])
        for key in attrs:
          if key in self.getObjAttrs().keys():
            for s_preview in [None,'preview']:
              req = {'preview':s_preview}
              ob = self.getObjVersion(req)
              if hasattr(ob,key):
                for s_lang in self.getLanguages():
                  attr = '%s_%s'%(key,s_lang)
                  value = getattr(ob,key)
                  setattr(ob,attr,value)
	    self.__delObjProperty__(key)

      # ---------------------------------------------------------------------
      # 26.02.2002
      # Build #067: Activity-Toggle.
      # ---------------------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '067':
        for s_preview in [None,'preview']:
          req = {'preview':s_preview}
          ob = self.getObjVersion(req)
          for s_key in ['active']:
            for s_lang in self.getLanguages():
              attr = '%s_%s'%(s_key,s_lang)
              if not hasattr(ob,attr) or getattr(self,'build','') <= '067':
                active = getattr(ob,'attr_active_start_%s'%s_lang,None) is not None or \
                         getattr(ob,'attr_active_end_%s'%s_lang,None) is not None 
                setattr(ob,attr,active)

      # Recurse through child-nodes.
      for ob in self.getChildNodes():
        message = message + ob.recurse_updateVersion(lang,manage_lang,REQUEST)
      
      # Return with message.
      return message

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getObjChildren
    # ---------------------------------------------------------------------------------------------
    def getObjChildren(self, id, REQUEST):
      obs = []
      for ob in self.getChildNodes(REQUEST):
        if len(id)==0 or ob.id[:len(id)]==id:
          obs.append(ob)
      return obs

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.initObjChild
    # ---------------------------------------------------------------------------------------------
    def initObjChild(self, id, _sort_id, type, REQUEST):

      ##### Create ####
      if self.dGlobalAttrs.has_key(type):
        obj = self.dGlobalAttrs[type]['obj_class'](id,_sort_id+1)
      else:
        obj = self.dGlobalAttrs['ZMSCustom']['obj_class'](id,_sort_id+1)
        obj.meta_id = type
      self._setObject(obj.id, obj)

      oItem = getattr(self,obj.id)
      ##### Object State ####
      oItem.setObjStateNew(REQUEST)
      ##### Init Properties ####
      for s_lang in self.getLanguages():
        oItem.setObjProperty('active',1,s_lang)
      ##### VersionManager ####
      oItem.onChangeObj(REQUEST)
          
      ##### Normalize Sort-IDs ####
      self.normalizeSortIds(_globals.id_prefix(id))
        
      return oItem
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.initObjChildren
    # ---------------------------------------------------------------------------------------------
    def initObjChildren(self, REQUEST):
      if _globals.debug: print "[%s.initObjChildren]: "%(self.meta_id)
      for obj_attr in self.getMetaobj(self.meta_id)['attrs']:
        type = obj_attr['type']
        if type in self.dGlobalAttrs.keys() or type in self.getMetaobjIds():
          id = obj_attr['id']
          repetitive = obj_attr.has_key('repetitive') and obj_attr['repetitive']==1
          if repetitive:
            if id in self.objectIds():
              new_id = self.getNewId(id)
              self.manage_renameObject(id=id,new_id=new_id)
              if _globals.debug: print "[%s.initObjChildren]: Rename %s to %s"%(self.meta_type,id,new_id)
          else:
            obj_children = self.getObjChildren(id,REQUEST)
            if not id in self.objectIds():
              if len(obj_children) > 0:
                self.manage_renameObject(id=obj_children[0].id,new_id=id)
                for i in range(1,len(obj_children)):
                  self.manage_delObjects(ids=[obj_children[i].id])
                if _globals.debug: print "[%s.initObjChildren]: Rename %s0 to %s"%(self.meta_type,id,id)
              else:
                self.initObjChild(id,0,type,REQUEST)
                if _globals.debug: print "[%s.initObjChildren]: Create %s"%(self.meta_type,id)


    """
    ###############################################################################################    
    #
    #   Eigenschaften
    #
    ###############################################################################################    
    """

    ###############################################################################################
    #  ZMSCustom.manage_changeProperties: 
    #
    #  Change Custom properties.
    ###############################################################################################
    def manage_changeProperties(self, lang, manage_lang, REQUEST, RESPONSE):
        """ ZMSCustom.manage_changeProperties """
        message = ''
        
        redirect_self = 0
        for attr in self.getMetaobj(self.meta_id)['attrs']:
           attr_type = attr['type']
           redirect_self = redirect_self or attr_type in self.dGlobalAttrs.keys()
           redirect_self = redirect_self or attr_type in self.getMetaobjIds()
        redirect_self = redirect_self and not REQUEST.get('btn','') == self.getLangStr('BTN_CANCEL',manage_lang)
           
        if REQUEST.get('btn','') != self.getLangStr('BTN_CANCEL',manage_lang):
          
          ##### DELETE OBJECT-CHILD ####
          if REQUEST.get('btn','') == 'delete':
            id = REQUEST['id']
            return self.manage_deleteObjs(lang,manage_lang,[id],REQUEST,RESPONSE)
          
          ##### CREATE OBJECT-CHILD ####
          elif REQUEST.get('btn','') == 'create':
            id = REQUEST['id'] + str(self.getSequence().nextVal())
            sort_id = self.getNewSortId()
            type = REQUEST['type']
            obj = self.initObjChild(id,sort_id,type,REQUEST)
            message = self.getLangStr('MSG_INSERTED',manage_lang)%obj.display_type(REQUEST)
            return RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(obj.id,lang,manage_lang,urllib.quote(message)))
          
          ##### MOVE-UP OBJECT-CHILD ####
          elif REQUEST.get('btn','') == 'move_up':
            id = REQUEST['id']
            ob = getattr(self,id)
            return ob.manage_moveObjUp(lang,manage_lang,REQUEST,RESPONSE)
          
          ##### MOVE-DOWN OBJECT-CHILD ####
          elif REQUEST.get('btn','') == 'move_down':
            id = REQUEST['id']
            ob = getattr(self,id)
            return ob.manage_moveObjDown(lang,manage_lang,REQUEST,RESPONSE)
          
          else:
            
            ##### Object State #####
            self.setObjStateModified(REQUEST)
            
            ##### Properties #####
            for key in self.getObjAttrs().keys():
              obj_attr = self.getObjAttr(key)
              if obj_attr['xml']:
                self.setReqProperty(key,REQUEST)
            
            ##### Blobs ####
            _blobfields.synchronizeBlobFields(self)
          
            ##### VersionManager ####
            self.onChangeObj(REQUEST)
            
            ##### Message #####
            message = self.getLangStr('MSG_CHANGED',manage_lang)

        # Return with message.
        if redirect_self:          
          return RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,urllib.quote(message)))
        else:        
          self.checkIn(REQUEST)
          return RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s#_%s'%(self.getParentNode().absolute_url(),lang,manage_lang,urllib.quote(message),self.id))


    # -------------------------------------------------------------------------------------------------
    #	ZMSCustom.isMetaType:
    # -------------------------------------------------------------------------------------------------
    def isMetaType(self, meta_type, REQUEST={}):
      value = 0
      # ZMSCustom.
      if type(meta_type) is type(''):
        value = value or (self.meta_type == meta_type or self.meta_id == meta_type)
      elif type(meta_type) == type([]):
        value = value or (self.meta_type in meta_type or self.meta_id in meta_type)
      # ZMSContainerObject.
      if not value:
        value = value or ZMSContainerObject.isMetaType(self,meta_type,REQUEST)
      return value
    
    # -------------------------------------------------------------------------------------------------
    #	ZMSCustom.isPage:
    # -------------------------------------------------------------------------------------------------
    def isPage(self): return self.getType() == 'ZMSDocument'

    # -------------------------------------------------------------------------------------------------
    #	ZMSObject.isPageElement:
    # -------------------------------------------------------------------------------------------------
    def isPageElement(self): return self.getType() == 'ZMSObject'

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getType
    # ---------------------------------------------------------------------------------------------
    def getType(self):
      docElmnt = self.getDocumentElement()
      meta_obj = docElmnt.getMetaobj(self.meta_id)
      return meta_obj['type']

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getDtmlTemplate
    # ---------------------------------------------------------------------------------------------
    def getDtmlTemplate(self):
      root = self.getDocumentElement().aq_parent
      meta_type = self.getType()
      if meta_type == 'ZMSObject':
        tmplt = '%s_%s'%(self.meta_type,self.meta_id)
      elif meta_type == 'ZMSDocument':
        tmplt = 'bodyContent%s_%s'%(self.meta_type,self.meta_id)
      elif meta_type == 'ZMSTeaserElement':
        tmplt = 'bodyContentTeaser%s_%s'%(self.meta_type,self.meta_id)
      dtml_tmplt = getattr(root,tmplt,None)
      return dtml_tmplt

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getTitlealt
    # ---------------------------------------------------------------------------------------------
    def getTitlealt(self,REQUEST):
      s_titleAlt = ""
      if self.isPage():
        try:
          attr = self.getDocumentElement().getMetaobj(self.meta_id)['attrs'][0]
          s_titleAlt = str(self.getObjProperty(attr['id'],REQUEST))
        except:
          pass
      else:
        try:
          for oAttr in self.getMetaobj(self.meta_id)['attrs']:
            if len(s_titleAlt) == 0:
              key = oAttr['id']
              datatype = oAttr['type']
              if datatype not in self.dGlobalAttrs.keys():
                value = self.getObjProperty(key,REQUEST)
                s_titleAlt = self.search_quote(self.getObjProperty(key,REQUEST),80,'')
        except:
          pass
      if len(s_titleAlt) == 0: 
        s_titleAlt = self.display_type(REQUEST)
      return s_titleAlt

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getTitle
    # ---------------------------------------------------------------------------------------------
    def getTitle(self,REQUEST): 
      s_title = ""
      if self.isPage():
        try:
          attr = self.getDocumentElement().getMetaobj(self.meta_id)['attrs'][1]
          s_title = str(self.getObjProperty(attr['id'],REQUEST))
        except:
          pass
      if len(s_title) == 0: 
        s_title = self.display_type(REQUEST)
      """  
      s_title = self.getTitlealt(REQUEST)
      s_display_type = self.display_type(REQUEST)
      if s_title != s_display_type:
        s_title = s_display_type + " " + s_title
      """  
      return s_title


    """
    ###############################################################################################    
    ###  
    ###  HTML-Darstellung 
    ### 
    ###############################################################################################    
    """

    # preload display interface for custom-objects
    render_default = HTMLFile('dtml/metaobj/render_default', globals()) 

    # ---------------------------------------------------------------------------------------------
    #  ZMSCustom._getBodyContent:
    #
    #  HTML presentation of Special Document. 
    # ---------------------------------------------------------------------------------------------
    def _getBodyContent(self, REQUEST, thumbnail=0):
      s = ""
      try:
        dtml_tmplt = self.getDtmlTemplate()
        if self.getType() == 'ZMSObject':
          s = s + dtml_tmplt(self,REQUEST)
        elif self.getType() == 'ZMSDocument':
          s = s + dtml_tmplt(self,REQUEST)
        elif self.getType() == 'ZMSTeaserElement':
          s = s + '<table cellspacing="0" cellpadding="0" border="0" width="100%" class="teaserElement">\n'
          s = s + '<tr'
          if REQUEST.has_key('bgcolor_text'):
            s = s + ' class="%s"'%str(REQUEST['bgcolor_text'])
          s = s + '>\n'
          s = s + '<td align="left" valign="top" class="teaserText">'
          s = s + '<p class="teaserText">%s</p>'%dtml_tmplt(self,REQUEST)
          s = s + '</td>\n'
          s = s + '</tr>\n'
          s = s + '</table>\n'
      except:
        s = s + '[Exception]:<br>'
        s = s + '-  Type: <strong>%s</strong><br>'%str(sys.exc_type)
        s = s + '-  Value: <strong>%s</strong><br>'%str(sys.exc_value)
        s = s + '-  Traceback: <strong>%s</strong>'%str(sys.exc_traceback)
      # Return <html>.
      return s

    # ---------------------------------------------------------------------------------------------
    #  ZMSCustom.renderShort:
    #
    #  Renders short presentation of Special Document.
    # ---------------------------------------------------------------------------------------------
    def renderShort(self, REQUEST):
      if self.getType() == 'ZMSDocument':
        html = self.render_default(self,ob=self,REQUEST=REQUEST)
      else:
        html = self._getBodyContent(REQUEST)
      # Return <html>.
      return html

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.printHtml:
    #
    #	Renders print presentation of Special Document.
    # ---------------------------------------------------------------------------------------------
    def printHtml(self, level, sectionize, sectionizer, REQUEST):
      html = self._getBodyContent(REQUEST)
      if len(html) == 0: 
        html = self.render_default(self,ob=self,REQUEST=REQUEST)
      # Return <html>.
      return html

###################################################################################################
